'use strict';

exports.__esModule = true;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * The TextMetrics object represents the measurement of a block of text with a specified style.
 *
 * ```js
 * let style = new PIXI.TextStyle({fontFamily : 'Arial', fontSize: 24, fill : 0xff1010, align : 'center'})
 * let textMetrics = PIXI.TextMetrics.measureText('Your text', style)
 * ```
 *
 * @class
 * @memberOf PIXI
 */
var TextMetrics = function () {
    /**
     * @param {string} text - the text that was measured
     * @param {PIXI.TextStyle} style - the style that was measured
     * @param {number} width - the measured width of the text
     * @param {number} height - the measured height of the text
     * @param {array} lines - an array of the lines of text broken by new lines and wrapping if specified in style
     * @param {array} lineWidths - an array of the line widths for each line matched to `lines`
     * @param {number} lineHeight - the measured line height for this style
     * @param {number} maxLineWidth - the maximum line width for all measured lines
     * @param {Object} fontProperties - the font properties object from TextMetrics.measureFont
     */
    function TextMetrics(text, style, width, height, lines, lineWidths, lineHeight, maxLineWidth, fontProperties) {
        _classCallCheck(this, TextMetrics);

        this.text = text;
        this.style = style;
        this.width = width;
        this.height = height;
        this.lines = lines;
        this.lineWidths = lineWidths;
        this.lineHeight = lineHeight;
        this.maxLineWidth = maxLineWidth;
        this.fontProperties = fontProperties;
    }

    /**
     * Measures the supplied string of text and returns a Rectangle.
     *
     * @param {string} text - the text to measure.
     * @param {PIXI.TextStyle} style - the text style to use for measuring
     * @param {boolean} [wordWrap] - optional override for if word-wrap should be applied to the text.
     * @param {HTMLCanvasElement} [canvas] - optional specification of the canvas to use for measuring.
     * @return {PIXI.TextMetrics} measured width and height of the text.
     */


    TextMetrics.measureText = function measureText(text, style, wordWrap) {
        var canvas = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : TextMetrics._canvas;

        wordWrap = wordWrap || style.wordWrap;
        var font = style.toFontString();
        var fontProperties = TextMetrics.measureFont(font);
        var context = canvas.getContext('2d');

        context.font = font;

        var outputText = wordWrap ? TextMetrics.wordWrap(text, style, canvas) : text;
        var lines = outputText.split(/(?:\r\n|\r|\n)/);
        var lineWidths = new Array(lines.length);
        var maxLineWidth = 0;

        for (var i = 0; i < lines.length; i++) {
            var lineWidth = context.measureText(lines[i]).width + (lines[i].length - 1) * style.letterSpacing;

            lineWidths[i] = lineWidth;
            maxLineWidth = Math.max(maxLineWidth, lineWidth);
        }
        var width = maxLineWidth + style.strokeThickness;

        if (style.dropShadow) {
            width += style.dropShadowDistance;
        }

        var lineHeight = style.lineHeight || fontProperties.fontSize + style.strokeThickness;
        var height = Math.max(lineHeight, fontProperties.fontSize + style.strokeThickness) + (lines.length - 1) * (lineHeight + style.leading);

        if (style.dropShadow) {
            height += style.dropShadowDistance;
        }

        return new TextMetrics(text, style, width, height, lines, lineWidths, lineHeight + style.leading, maxLineWidth, fontProperties);
    };

    /**
     * @return {number} enumeration of supported language options for word wrap
     */


    /**
     * @param {string} c character to tentatively begin the line
     * @param {number} language the language to check line-break rules against
     * @return {boolean}  whether the specified character can begin a line in the specified language
     */
    TextMetrics.canBeginLineCJK = function canBeginLineCJK(c, language) {
        if (!c || !language || language === TextMetrics.CJKLanguage.OTHER) {
            return true;
        }

        var notPermittedToBeginLine = void 0;

        if (language === TextMetrics.CJKLanguage.CHINESE) {
            // Simplified Chinese
            // !%),.:;?]}¢°·’""†‡›℃∶、。〃〆〕〗〞﹚﹜！＂％＇），．：；？！］｝～
            // Traditional Chinese
            // !),.:;?]}¢·–— ’"•" 、。〆〞〕〉》」︰︱︲︳﹐﹑﹒﹓﹔﹕﹖﹘﹚﹜！），．：；？︶︸︺︼︾﹀﹂﹗］｜｝､
            // Merging the two rules into one since it's impossible to tell whether
            // the word is simplified or traditional via unicode
            notPermittedToBeginLine = '!%),.:;?]}\xA2' + '\xB0\xB7\u2019"\u2020\u2021\u203A\u2103\u2236\u3001\u3002\u3003\u3006\u3015' + '\u3017\u301E\uFE5A\uFE5C\uFF01\uFF02\uFF05\uFF07\uFF09\uFF0C\uFF0E\uFF1A\uFF1B\uFF1F\uFF01' + '\uFF3D\uFF5D\uFF5E\u2014\u2022\u3009\u300B\u300D\uFE30\uFE31\uFE32\uFE33\uFE50\uFE51\uFE52\uFE53' + '\uFE54\uFE55\uFE56\uFE58\uFE36\uFE38\uFE3A\uFE3C\uFE3E\uFE40\uFE42\uFE57\uFF5C\uFF64';
        } else if (language === TextMetrics.CJKLanguage.JAPANESE) {
            // )]｝〕〉》」』】〙〗〟’"｠»ヽヾーァィゥェォッャュョヮヵヶぁぃぅぇぉっゃゅょゎゕゖㇰㇱㇲㇳㇴㇵㇶ
            // ㇷㇸㇹㇺㇻㇼㇽㇾㇿ々〻‐゠–〜?!‼⁇⁈⁉・、:;,。.
            notPermittedToBeginLine = ')]\uFF5D\u3015\u3009\u300B\u300D\u300F' + '\u3011\u3019\u3017\u301F\u2019"\uFF60\xBB\u30FD\u30FE\u30FC\u30A1\u30A3' + '\u30A5\u30A7\u30A9\u30C3\u30E3\u30E5\u30E7\u30EE\u30F5\u30F6\u3041\u3043\u3045\u3047' + '\u3049\u3063\u3083\u3085\u3087\u308E\u3095\u3096\u31F0\u31F1\u31F2\u31F3\u31F4\u31F5' + '\u31F6\u31F7\u31F8\u31F9\u31FA\u31FB\u31FC\u31FD\u31FE\u31FF\u3005\u303B\u2010\u30A0' + '\u2013\u301C?!\u203C\u2047\u2048\u2049\u30FB\u3001:;,\u3002.';
        } else if (language === TextMetrics.CJKLanguage.KOREAN) {
            // !%),.:;?]}¢°’"†‡℃〆〈《「『〕！％），．：；？］｝
            notPermittedToBeginLine = '!%),.:;?]' + '}\xA2\xB0\u2019"\u2020\u2021\u2103\u3006\u3008\u300A\u300C\u300E' + '\u3015\uFF01\uFF05\uFF09\uFF0C\uFF0E\uFF1A\uFF1B\uFF1F\uFF3D\uFF5D';
        }

        return notPermittedToBeginLine.indexOf(c) === -1;
    };

    /**
     * @param {string} c character to tentatively end the line
     * @param {number} language the language to check line-break rules against
     * @return {boolean}  whether the specified character can end a line in the specified language
     */


    TextMetrics.canEndLineCJK = function canEndLineCJK(c, language) {
        if (!c || !language || language === TextMetrics.CJKLanguage.OTHER) {
            return true;
        }

        var notPermittedToEndLine = void 0;

        if (language === TextMetrics.CJKLanguage.CHINESE) {
            // Simplified Chinese
            // $(£¥·‘"〈《「『【〔〖〝﹙﹛＄（．［｛￡￥
            // Traditional Chinese
            // ([{£¥‘"‵〈《「『〔〝︴﹙﹛（｛︵︷︹︻︽︿﹁﹃﹏
            // Merging the two rules into one since it's impossible to tell whether the word is simplified
            // or traditional via unicode
            notPermittedToEndLine = '$(\xA3\xA5\xB7\u2018"\u3008\u300A\u300C' + '\u300E\u3010\u3014\u3016\u301D\uFE59\uFE5B\uFF04\uFF08\uFF0E\uFF3B\uFF5B\uFFE1\uFFE5' + '[{\u2035\uFE34\uFE35\uFE37\uFE39\uFE3B\uFE3D\uFE3F\uFE41\uFE43\uFE4F';
        } else if (language === TextMetrics.CJKLanguage.JAPANESE) {
            // ([｛〔〈《「『【〘〖〝‘"｟«
            notPermittedToEndLine = '([\uFF5B\u3014\u3008\u300A\u300C\u300E' + '\u3010\u3018\u3016\u301D\u2018"\uFF5F\xAB';
        } else if (language === TextMetrics.CJKLanguage.KOREAN) {
            // $([\{£¥‘"々〇〉》」〔＄（［｛｠￥￦ #
            notPermittedToEndLine = '$([\\{\xA3\xA5\u2018"' + '\u3005\u3007\u3009\u300B\u300D\u3014\uFF04\uFF08\uFF3B\uFF5B\uFF60\uFFE5\uFFE6#';
        }

        return notPermittedToEndLine.indexOf(c) === -1;
    };

    /**
     * @param {string} c1 the first character, to tentatively end this line
     * @param {number} c2 the second character, to tentatively begin the next line
     * @return {boolean} whether the two characters can be split between two lines according to
     * Japanese line-breaking conventions
     */


    TextMetrics.canBeSplitJapanese = function canBeSplitJapanese(c1, c2) {
        if (!c1 || !c2) {
            return true;
        }

        var doNotSplit = [['\u2014', '\u2026'], ['\u2025', '\u3033'], ['\u3034', '\u3035']];

        for (var i = 0; i < doNotSplit.length; i++) {
            var doNotSplitPair = doNotSplit[i];

            if (doNotSplitPair[0] === c1 && doNotSplitPair[1] === c2) {
                return false;
            }
        }

        // Ignoring grouped ruby characters --
        // we can't tell which ruby characters corresponds to which kanji.

        return isNaN(c1) || isNaN(c2);
    };

    /**
     * @param {string} word glean the language from this word
     * @return {number} Korean, Chinese, Japanese, or Other (for any other language)
     */


    TextMetrics.CJKTextLanguage = function CJKTextLanguage(word) {
        // From https://unicode-table.com/en/ :
        // 1100 - 11FF Hangul Jamo (Korean)
        // 2E80 - 2EFF CJK Radicals Supplement (CJK)
        // 2F00 - 2FDF Kangxi Radicals (Chinese)
        // 3000 - 303F CJK Symbols and Punctuation (CJK)
        // 3040 - 309F Hiragana (Japanese)
        // 30A0 - 30FF Katakana (Japanese)
        // 3100 - 312F Bopomofo (Chinese)
        // 3130 - 318F Hangul Compatibility Jamo (Korean)
        // 3190 - 319F Kanbun (Chinese, Japanese)
        // 31A0 - 31BF Bopomofo Extended (Chinese)
        // 31C0 - 31EF CJK Strokes (Chinese)
        // 31F0 - 31FF Katakana phonetic extensions (Japanese)
        // 3200 - 32FF Enclosed CJK letters and Months (Chinese)
        // 3300 - 33FF CJK Compatibility (CJK)
        // 3400 - 4DBF CJK Unified Ideographs Extension A (CJK)
        // 4DC0 - 4DFF Yijing Hexagram Symbols (Chinese)
        // 4E00 - 9FAF CJK unified ideographs (CJK)
        // AC00 - D7AF Hangul Syllables (Korean)
        // A960 - A97F Hangul Jamo Extended A (Korean)
        // D7B0 - D7FF Hangul Jamo Extended B (Korean)
        // FE30 - FE4F CJK Compatibility forms (CJK)
        // FF00 - FF9F Halfwidth and fullwidth forms (CJK)
        // 20000 - 2A6DF CJK Unified Ideographs Extension B (CJK)
        // 2A700 - 2B73F CJK Unified Ideographs Extension C (CJK)
        // 2B740 - 2B81F CJK Unified Ideographs Extension D (CJK)
        // 2B820 - 2CEAF CJK Unified Ideographs Extension E (CJK)
        // 2CEB0 - 2EBEF CJK Unified Ideographs Extension F (CJK)
        // 2F800 - 2FA1F CJK Compatibility Ideographs Supplement (CJK)

        var jpnRegExp = new RegExp('[\\u3040-\\u309f\\u30a0-\\u30ff\\u31f0-\\u31ff]');
        var jpnExclusiveCharacters = word.match(jpnRegExp);

        if (jpnExclusiveCharacters) {
            return TextMetrics.CJKLanguage.JAPANESE;
        }

        var krnRegExp = new RegExp('[\\u1100-\\u11ff\\u3130-\\u318f\\uac00-\\ud7af' + '\\ua960-\\ua97f\\ud7b0-\\ud7ff]');
        var krnExclusiveCharacters = word.match(krnRegExp);

        if (krnExclusiveCharacters) {
            return TextMetrics.CJKLanguage.KOREAN;
        }

        var chnRegExp = new RegExp('[\\u2f00-\\u2fdf\\u3100-\\u312f\\u31a0-\\u31bf' + '\\u31c0-\\u31ef\\u3200-\\u32ff\\u4dc0-\\u4dff]');
        var chnExclusiveCharacters = word.match(chnRegExp);

        if (chnExclusiveCharacters) {
            return TextMetrics.CJKLanguage.CHINESE;
        }

        var cjkRegExp = new RegExp('[\\u2e80-\\u2eff\\u3000-\\u303f\\u3190-\\u319f' + '\\u3300-\\u33ff\\u3400-\\u4dbf\\u4e00-\\u9faf\\ufe30-\\ufe4f\\uff00-\\uff9f' + '\\u{20000}-\\u{2a6df}\\u{2a700}-\\u{2b73f}\\u{2b740}-\\u{2b81f}\\u{2b820}-\\u{2ceaf}' + '\\u{2ceb0}-\\u{2ebef}\\u{2f800}-\\u{2fa1f}]', 'u');

        var cjkCharacters = word.match(cjkRegExp);

        // assumption is that if we have no japan or korean exclusive characters
        // and we have CJK characters, that would be chinese text.
        if (cjkCharacters) {
            return TextMetrics.CJKLanguage.CHINESE;
        }

        return TextMetrics.CJKLanguage.OTHER;
    };

    /**
     * @param {string} character The character to get the width of
     * @param {string[]} characterCache A cache of pre-measured character widths
     * @param {function} measureText A function for determining character widths
     * @return {number} The width of the character according to cache / measureText
     */


    TextMetrics.getCharacterWidth = function getCharacterWidth(character, characterCache, measureText) {
        if (characterCache[character] === undefined) {
            characterCache[character] = measureText(character).width;
        }

        return characterCache[character];
    };

    /**
     *
     * @param {string} word The word which we are wrapping
     * @param {function} measureText The function used to measure character widths
     * @param {string[]} characterCache Mapping of characters to their widths already computed
     * @param {number} wordWrapWidth The maximum width a line can be
     * @return {string} The resulting CJK-word-wrapped string with line breaks
     */


    TextMetrics.CJKWordWrap = function CJKWordWrap(word, measureText, characterCache, wordWrapWidth) {
        var characters = word.split('');
        var wordLanguage = TextMetrics.CJKTextLanguage(word);
        var totalCharacterWidth = 0;
        var tooWideIndex = 0;
        var result = '';

        for (var startIndex = 0; startIndex < characters.length;) {
            for (tooWideIndex = startIndex; tooWideIndex < characters.length; tooWideIndex++) {
                totalCharacterWidth += TextMetrics.getCharacterWidth(characters[tooWideIndex], characterCache, measureText);

                if (totalCharacterWidth > wordWrapWidth) {
                    break;
                }
            }

            // This can happen once we've already broken up the word and started re-counting the length
            if (tooWideIndex === characters.length) {
                // we couldn't find a spot that made it too long, just add all of it
                result += word.substring(startIndex);
                break;
            }

            var lineBreakIndex = 0;

            for (lineBreakIndex = tooWideIndex; lineBreakIndex > startIndex; lineBreakIndex--) {
                // find the latest point we can break cleanly
                // according to https://en.wikipedia.org/wiki/Line_breaking_rules_in_East_Asian_languages

                var c1 = lineBreakIndex >= startIndex ? characters[lineBreakIndex - 1] : null;
                var c2 = characters[lineBreakIndex];

                if (TextMetrics.canBeginLineCJK(c2, wordLanguage) && TextMetrics.canEndLineCJK(c1, wordLanguage) && (wordLanguage !== TextMetrics.CJKLanguage.JAPANESE || TextMetrics.canBeSplitJapanese(c1, c2))) {
                    break;
                }
            }
            if (lineBreakIndex === 0) {
                // couldn't find a suitable place to break, so use the latest point
                lineBreakIndex = tooWideIndex;
            }
            result += word.substring(startIndex, lineBreakIndex);
            result += '\n' + characters[lineBreakIndex];

            totalCharacterWidth = TextMetrics.getCharacterWidth(characters[lineBreakIndex], characterCache, measureText);

            startIndex = lineBreakIndex + 1;
        }

        return result;
    };

    /**
     * Applies newlines to a string to have it optimally fit into the horizontal
     * bounds set by the Text object's wordWrapWidth property.
     *
     * @private
     * @param {string} text - String to apply word wrapping to
     * @param {PIXI.TextStyle} style - the style to use when wrapping
     * @param {HTMLCanvasElement} [canvas] - optional specification of the canvas to use for measuring.
     * @return {string} New string with new lines applied where required
     */


    TextMetrics.wordWrap = function wordWrap(text, style) {
        var canvas = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : TextMetrics._canvas;

        var context = canvas.getContext('2d');

        // Greedy wrapping algorithm that will wrap words as the line grows longer
        // than its horizontal bounds.
        var result = '';
        var firstChar = text.charAt(0);
        var lines = text.split('\n');
        var wordWrapWidth = style.wordWrapWidth;
        var characterCache = {};

        for (var i = 0; i < lines.length; i++) {
            var spaceLeft = wordWrapWidth;
            var words = lines[i].split(' ');

            for (var j = 0; j < words.length; j++) {
                var wordWidth = context.measureText(words[j]).width;

                var wordLanguage = TextMetrics.CJKTextLanguage(words[j]);

                if (style.breakWords && wordWidth > wordWrapWidth) {
                    // Word should be split in the middle
                    var characters = words[j].split('');

                    if (wordLanguage === TextMetrics.CJKLanguage.OTHER) {
                        for (var c = 0; c < characters.length; c++) {
                            var character = characters[c];

                            var characterWidth = TextMetrics.getCharacterWidth(character, characterCache, context.measureText.bind(context));

                            if (characterWidth > spaceLeft) {
                                result += '\n' + character;
                                spaceLeft = wordWrapWidth - characterWidth;
                            } else {
                                /* eslint-disable max-depth */
                                if (c === 0 && (j > 0 || firstChar === ' ')) {
                                    result += ' ';
                                }
                                result += character;
                                spaceLeft -= characterWidth;
                            }
                        }
                    } else {
                        result += TextMetrics.CJKWordWrap(words[j], context.measureText.bind(context), characterCache, wordWrapWidth);
                    }
                } else {
                    var wordWidthWithSpace = wordWidth + context.measureText(' ').width;

                    if (j === 0 || wordWidthWithSpace > spaceLeft) {
                        // Skip printing the newline if it's the first word of the line that is
                        // greater than the word wrap width.
                        if (j > 0) {
                            result += '\n';
                        }
                        result += words[j];
                        spaceLeft = wordWrapWidth - wordWidth;
                    } else {
                        spaceLeft -= wordWidthWithSpace;
                        result += ' ' + words[j];
                    }
                }
            }

            if (i < lines.length - 1) {
                result += '\n';
            }
        }

        return result;
    };

    /**
     *  Convienience function for logging each line added
     *  during the wordWrap method
     *
     * @param  {string}   line    - The line of text to add
     * @param  {boolean}  newLine - Add new line character to end
     * @return {string}   A formatted line
     */


    TextMetrics.addLine = function addLine(line) {
        var newLine = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

        line = newLine ? line + '\n' : line;

        return line;
    };

    /**
     * Gets & sets the widths of calculated characters in a cache object
     *
     * @param  {string}                    key            The key
     * @param  {number}                    letterSpacing  The letter spacing
     * @param  {object}                    cache          The cache
     * @param  {CanvasRenderingContext2D}  context        The canvas context
     * @return {number}                    The from cache.
     */


    TextMetrics.getFromCache = function getFromCache(key, letterSpacing, cache, context) {
        var width = cache[key];

        if (width === undefined) {
            var spacing = key.length * letterSpacing;

            width = context.measureText(key).width + spacing;
            cache[key] = width;
        }

        return width;
    };

    /**
     * Calculates the ascent, descent and fontSize of a given font-style
     *
     * @static
     * @param {string} font - String representing the style of the font
     * @return {PIXI.TextMetrics~FontMetrics} Font properties object
     */


    TextMetrics.measureFont = function measureFont(font) {
        // as this method is used for preparing assets, don't recalculate things if we don't need to
        if (TextMetrics._fonts[font]) {
            return TextMetrics._fonts[font];
        }

        var properties = {};

        var canvas = TextMetrics._canvas;
        var context = TextMetrics._context;

        context.font = font;

        var width = Math.ceil(context.measureText('|MÉq').width);
        var baseline = Math.ceil(context.measureText('M').width);
        var height = 2 * baseline;

        baseline = baseline * 1.4 | 0;

        canvas.width = width;
        canvas.height = height;

        context.fillStyle = '#f00';
        context.fillRect(0, 0, width, height);

        context.font = font;

        context.textBaseline = 'alphabetic';
        context.fillStyle = '#000';
        context.fillText('|MÉq', 0, baseline);

        var imagedata = context.getImageData(0, 0, width, height).data;
        var pixels = imagedata.length;
        var line = width * 4;

        var i = 0;
        var idx = 0;
        var stop = false;

        // ascent. scan from top to bottom until we find a non red pixel
        for (i = 0; i < baseline; ++i) {
            for (var j = 0; j < line; j += 4) {
                if (imagedata[idx + j] !== 255) {
                    stop = true;
                    break;
                }
            }
            if (!stop) {
                idx += line;
            } else {
                break;
            }
        }

        properties.ascent = baseline - i;

        idx = pixels - line;
        stop = false;

        // descent. scan from bottom to top until we find a non red pixel
        for (i = height; i > baseline; --i) {
            for (var _j = 0; _j < line; _j += 4) {
                if (imagedata[idx + _j] !== 255) {
                    stop = true;
                    break;
                }
            }

            if (!stop) {
                idx -= line;
            } else {
                break;
            }
        }

        properties.descent = i - baseline;
        properties.fontSize = properties.ascent + properties.descent;

        TextMetrics._fonts[font] = properties;

        return properties;
    };

    _createClass(TextMetrics, null, [{
        key: 'CJKLanguage',
        get: function get() {
            return {
                OTHER: 0,
                CHINESE: 1,
                JAPANESE: 2,
                KOREAN: 3
            };
        }
    }]);

    return TextMetrics;
}();

/**
 * Internal return object for {@link PIXI.TextMetrics.measureFont `TextMetrics.measureFont`}.
 * @class FontMetrics
 * @memberof PIXI.TextMetrics~
 * @property {number} ascent - The ascent distance
 * @property {number} descent - The descent distance
 * @property {number} fontSize - Font size from ascent to descent
 */

exports.default = TextMetrics;
var canvas = document.createElement('canvas');

canvas.width = canvas.height = 10;

/**
 * Cached canvas element for measuring text
 * @memberof PIXI.TextMetrics
 * @type {HTMLCanvasElement}
 * @private
 */
TextMetrics._canvas = canvas;

/**
 * Cache for context to use.
 * @memberof PIXI.TextMetrics
 * @type {CanvasRenderingContext2D}
 * @private
 */
TextMetrics._context = canvas.getContext('2d');

/**
 * Cache of PIXI.TextMetrics~FontMetrics objects.
 * @memberof PIXI.TextMetrics
 * @type {Object}
 * @private
 */
TextMetrics._fonts = {};
//# sourceMappingURL=TextMetrics.js.map